@@ -146,7 +146,7 @@ static BOOL get_num_records_hook( EVENTLOG_INFO * info )
 
 	/* lock the tdb since we have to get 2 records */
 
-	tdb_lock_bystring( ELOG_TDB_CTX(info->etdb), EVT_NEXT_RECORD, 1 );
+	tdb_lock_bystring_with_timeout( ELOG_TDB_CTX(info->etdb), EVT_NEXT_RECORD, 1 );
 	next_record = tdb_fetch_int32( ELOG_TDB_CTX(info->etdb), EVT_NEXT_RECORD);
 	oldest_record = tdb_fetch_int32( ELOG_TDB_CTX(info->etdb), EVT_OLDEST_ENTRY);
 	tdb_unlock_bystring( ELOG_TDB_CTX(info->etdb), EVT_NEXT_RECORD);
@@ -289,12 +289,10 @@ Eventlog_entry *get_eventlog_record( prs_struct * ps, TDB_CONTEXT * tdb,
 	int srecno;
 	int reclen;
 	int len;
-	uint8 *rbuff;
 
 	pstring *wpsource, *wpcomputer, *wpsid, *wpstrs, *puserdata;
 
 	key.dsize = sizeof( int32 );
-	rbuff = NULL;
 
 	srecno = recno;
 	key.dptr = ( char * ) &srecno;
@@ -682,6 +680,10 @@ NTSTATUS _eventlog_read_eventlog( pipes_struct * p,
 	int bytes_left, record_number;
 	uint32 elog_read_type, elog_read_dir;
 
+	if (info == NULL) {
+		return NT_STATUS_INVALID_HANDLE;
+	}
+
 	info->flags = q_u->flags;
 	ps = &p->out_data.rdata;
 
@@ -768,6 +770,10 @@ NTSTATUS _eventlog_get_oldest_entry( pipes_struct * p,
 {
 	EVENTLOG_INFO *info = find_eventlog_info_by_hnd( p, &q_u->handle );
 
+	if (info == NULL) {
+		return NT_STATUS_INVALID_HANDLE;
+	}
+
 	if ( !( get_oldest_entry_hook( info ) ) )
 		return NT_STATUS_ACCESS_DENIED;
 
@@ -785,6 +791,10 @@ NTSTATUS _eventlog_get_num_records( pipes_struct * p,
 {
 	EVENTLOG_INFO *info = find_eventlog_info_by_hnd( p, &q_u->handle );
 
+	if (info == NULL) {
+		return NT_STATUS_INVALID_HANDLE;
+	}
+
 	if ( !( get_num_records_hook( info ) ) )
 		return NT_STATUS_ACCESS_DENIED;
 
